package interval;

import java.util.Arrays;
import java.util.Collections;

public class Interval {
  /**
   * the lower bound of the interval
   */
  protected Integer low;

  /**
   * the upper bound of the interval
   */
  protected Integer high;

  /**
   * This method acts as constructor for an interval.
   *
   * @param low the lower bound
   * @param high the upper bound
   */
  public Interval(Integer low, Integer high) {
    this.low  = low;

    this.high = high;

    if(low == null || high == null) {
    	throw new IllegalArgumentException("neither the lower bound nor the upper bound may be null!");
    }

    if(low > high) {
      throw new IllegalArgumentException("the lower bound cannot be greater than the upper bound!");
    }
  }

  /**
   * This method acts as constructor for a single-value interval.
   *
   * @param value for the lower and upper bound
   */
  public Interval(Integer value) {
    this(value, value);
  }

  /**
   * This method creates a new interval instance representing the union of this interval with another interval.
   *
   * The lower bound and upper bound of the new interval is the minimum of both lower bounds and the maximum of both upper bounds, respectively.
   *
   * @param other the other interval
   * @return the new interval with the respective bounds
   */
  public Interval union(Interval other) {
      return new Interval(Math.min(low, other.low), Math.max(high, other.high));
  }

  /**
   * This method creates a new interval instance representing the intersection of this interval with another interval.
   *
   * The lower bound and upper bound of the new interval is the maximum of both lower bounds and the minimum of both upper bounds, respectively.
   *
   * @param other the other interval
   * @return the new interval with the respective bounds
   */
  public Interval intersect(Interval other) {
    Interval interval = null;

    if(this.intersects(other)) {
      interval = new Interval(Math.max(low, other.low), Math.min(high, other.high));
    }

    return interval;
  }

  /**
   * This method determines if this interval intersects with another interval.
   *
   * @param other the other interval
   * @return true if the intervals intersect, else false
   */
  public boolean intersects(Interval other) {
      return (low >= other.low && low <= other.high)
        || (high >= other.low && high <= other.high)
        || (low <= other.low && high >= other.high);
  }

  /**
   * This method determines if this interval contains another interval.
   *
   * The method still returns true, if the borders match. An empty interval does not contain any interval and is not contained in any interval either. So if the callee or parameter is an empty interval, this method will return false.
   *
   * @param other the other interval
   * @return true if this interval contains the other interval, else false
   */
  public boolean contains(Interval other) {
     return (low <= other.low && other.high <= high);
  }

  /**
   * This method multiplies this interval with another interval.
   *
   * @param other interval to multiply this interval with
   * @return new interval that represents the multiplication of the two intervals
   */
  public Interval times(Interval other) {
	  Integer[] values = {
                      low.intValue() * other.low.intValue(),
                      low.intValue() * other.high.intValue(),
                      high.intValue() * other.low.intValue(),
                      high.intValue() * other.high.intValue()
                    };

    return new Interval(Collections.min(Arrays.asList(values)), Collections.max(Arrays.asList(values)));
  }

  /* (non-Javadoc)
   * @see java.lang.Object#equals(java.lang.Object)
   */
  @Override
  public boolean equals(Object other) {

    if (other == this) {
      return true;
    }

    else if (other == null) {
      return false;
    }

    else if (!this.getClass().equals(other.getClass())) {
      return false;
    }

    else {
      Interval another = (Interval)other;

      return low.equals(another.low) && high.equals(another.high);
    }
  }

  /* (non-Javadoc)
   * @see java.lang.Object#hashCode()
   */
  @Override
  public int hashCode() {
    int result = 17;

    result = 31 * result + low.hashCode();
    result = 31 * result + high.hashCode();

    return result;
  }

  /* (non-Javadoc)
   * @see java.lang.Object#toString()
   */
  @Override
  public String toString() {
    return "[" + low + "; " + high + "]";
  }
}
